@@ -39,6 +39,8 @@
 #include <stdlib.h>
 #include <limits.h>
 
+#include <glib.h>
+
 #include "xferstats.h"
 
 typedef struct
@@ -49,38 +51,54 @@ typedef struct
 } config_function;
 
 /* prototypes for config functions */
-void config_logfile(pointers_t *, char *);
-void config_anon(pointers_t *);
-void config_guest(pointers_t *);
-void config_real(pointers_t *);
-void config_inbound(pointers_t *);
-void config_outbound(pointers_t *);
-void config_hourly(pointers_t *);
-void config_dow(pointers_t *);
-void config_dom(pointers_t *);
-void config_tld(pointers_t *);
-void config_domain(pointers_t *);
-void config_host(pointers_t *);
-void config_dir(pointers_t *);
-void config_file(pointers_t *);
-void config_monthly(pointers_t *);
-void config_html(pointers_t *);
-void config_stdin(pointers_t *);
-void config_logtype(pointers_t *, char *);
-void config_dir_depth(pointers_t *, char *);
-void config_dir_filter(pointers_t *, char *);
+static void config_logfile(pointers_t *, char *);
+static void config_anon(pointers_t *);
+static void config_guest(pointers_t *);
+static void config_real(pointers_t *);
+static void config_inbound(pointers_t *);
+static void config_outbound(pointers_t *);
+static void config_hourly(pointers_t *);
+static void config_dow(pointers_t *);
+static void config_dom(pointers_t *);
+static void config_tld(pointers_t *);
+static void config_domain(pointers_t *);
+static void config_host(pointers_t *);
+static void config_dir(pointers_t *);
+static void config_file(pointers_t *);
+static void config_monthly(pointers_t *);
+static void config_size(pointers_t *);
+static void config_html(pointers_t *, char *);
+static void config_stdin(pointers_t *);
+static void config_logtype(pointers_t *, char *);
+static void config_dir_depth(pointers_t *, char *);
+static void config_dir_filter(pointers_t *, char *);
 /* void config_tld_filter(pointers_t *, char *); */
 /* This will have to wait for another version */
-void config_number_file_stats(pointers_t *, char *);
-void config_number_dir_stats(pointers_t *, char *);
-void config_number_daily_stats(pointers_t *, char *);
-void config_number_tld_stats(pointers_t *, char *);
-void config_number_domain_stats(pointers_t *, char *);
-void config_number_host_stats(pointers_t *, char *);
-void config_max_report_size(pointers_t *, char *);
-void config_graph_path(pointers_t *, char *);
-void config_no_html_headers(pointers_t *);
-void config_single_page(pointers_t *);
+static void config_number_file_stats(pointers_t *, char *);
+static void config_number_dir_stats(pointers_t *, char *);
+static void config_number_daily_stats(pointers_t *, char *);
+static void config_number_tld_stats(pointers_t *, char *);
+static void config_number_domain_stats(pointers_t *, char *);
+static void config_number_host_stats(pointers_t *, char *);
+static void config_max_report_size(pointers_t *, char *);
+static void config_graph_path(pointers_t *, char *);
+static void config_no_html_headers(pointers_t *);
+static void config_single_page(pointers_t *);
+static void config_link_prefix(pointers_t *, char *);
+static void config_additive_db(pointers_t *, char *);
+static void config_strip_prefix(pointers_t *, char *);
+static void config_file_sort_pref(pointers_t *, char *);
+static void config_dir_sort_pref(pointers_t *, char *);
+static void config_domain_sort_pref(pointers_t *, char *);
+static void config_tld_sort_pref(pointers_t *, char *);
+static void config_host_sort_pref(pointers_t *, char *);
+static void config_dom_sort_pref(pointers_t *, char *);
+static void config_dow_sort_pref(pointers_t *, char *);
+static void config_hourly_sort_pref(pointers_t *, char *);
+static void config_daily_sort_pref(pointers_t *, char *);
+static void config_size_sort_pref(pointers_t *, char *);
+static void config_monthly_sort_pref(pointers_t *, char *);
+static void config_strict_check(pointers_t *);
 
 static config_function config_commands[] =
 {
@@ -99,7 +117,8 @@ static config_function config_commands[] =
   {"DIR_REPORT", (void *)config_dir, 0},
   {"FILE_REPORT", (void *)config_file, 0},
   {"MONTHLY_REPORT", (void *)config_monthly, 0},
-  {"HTML_OUTPUT", (void *)config_html, 0},
+  {"SIZE_REPORT", (void *)config_size, 0},
+  {"HTML_OUTPUT", (void *)config_html, 1},
   {"USE_STDIN", (void *)config_stdin, 0},
   {"LOG_TYPE", (void *)config_logtype, 1},
   {"DIR_DEPTH", (void *)config_dir_depth, 1},
@@ -116,151 +135,183 @@ static config_function config_commands[] =
   {"GRAPH_PATH", (void *)config_graph_path, 1},
   {"NO_HTML_HEADERS", (void *)config_no_html_headers, 0},
   {"SINGLE_PAGE", (void *)config_single_page, 0},
+  {"LINK_PREFIX", (void *)config_link_prefix, 1},
+  {"ADDITIVE_DB", (void *)config_additive_db, 1},
+  {"STRIP_PREFIX", (void *)config_strip_prefix, 1},
+  {"FILE_SORT_PREF", (void *)config_file_sort_pref, 1},
+  {"DIR_SORT_PREF", (void *)config_dir_sort_pref, 1},
+  {"DOMAIN_SORT_PREF", (void *)config_domain_sort_pref, 1},
+  {"TLD_SORT_PREF", (void *)config_tld_sort_pref, 1},
+  {"HOST_SORT_PREF", (void *)config_host_sort_pref, 1},
+  {"DOM_SORT_PREF", (void *)config_dom_sort_pref, 1},
+  {"DOW_SORT_PREF", (void *)config_dow_sort_pref, 1},
+  {"HOURLY_SORT_PREF", (void *)config_hourly_sort_pref, 1},
+  {"DAILY_SORT_PREF", (void *)config_daily_sort_pref, 1},
+  {"SIZE_SORT_PREF", (void *)config_size_sort_pref, 1},
+  {"MONTHLY_SORT_PREF", (void *)config_monthly_sort_pref, 1},
+  {"STRICT_CHECK", (void *)config_strict_check, 0},
   {"", NULL, 0}
 };
 
 
-void
-config_logfile(pointers_t *pointers, char *logfile)
+static void
+config_logfile(pointers_t *pointers, char * logfile)
 {
-  if (pointers->config->file_name != NULL)
-    free(pointers->config->file_name);
-
-  pointers->config->file_name = (char *)malloc(strlen(logfile) + 1);
-  strcpy(pointers->config->file_name, logfile);
+  pointers->config->logfiles = g_slist_append(pointers->config->logfiles,
+					      g_strdup(logfile));
 } /* config_logfile */
 
 
-void
+static void
 config_anon(pointers_t *pointers)
 {
   pointers->config->anon_traffic = 1;
 } /* config_anon */
 
 
-void
+static void
 config_guest(pointers_t *pointers)
 {
   pointers->config->guest_traffic = 1;
 } /* config_guest */
 
 
-void
+static void
 config_real(pointers_t *pointers)
 {
   pointers->config->real_traffic = 1;
 } /* config_real */
 
 
-void
+static void
 config_inbound(pointers_t *pointers)
 {
   pointers->config->inbound = 1;
 } /* config_inbound */
 
 
-void
+static void
 config_outbound(pointers_t *pointers)
 {
   pointers->config->outbound = 1;
 } /* config_outbound */
 
 
-void
+static void
 config_hourly(pointers_t *pointers)
 {
   pointers->config->hourly_traffic = 1;
 } /* config_hourly */
 
 
-void
+static void
 config_dow(pointers_t *pointers)
 {
   pointers->config->dow_traffic = 1;
 } /* config_dow */
 
 
-void
+static void
 config_dom(pointers_t *pointers)
 {
   pointers->config->dom_traffic = 1;
 } /* config_dom */
 
 
-void
+static void
 config_tld(pointers_t *pointers)
 {
   pointers->config->tld_traffic = 1;
 } /* config_tld */
 
 
-void
+static void
 config_domain(pointers_t *pointers)
 {
   pointers->config->domain_traffic = 1;
 } /* config_domain */
 
 
-void
+static void
 config_host(pointers_t *pointers)
 {
   pointers->config->host_traffic = 1;
 } /* config_host */
 
 
-void
+static void
 config_dir(pointers_t *pointers)
 {
   pointers->config->dir_traffic = 1;
 } /* config_dir */
 
 
-void
+static void
 config_file(pointers_t *pointers)
 {
   pointers->config->file_traffic = 1;
 } /* config_file */
 
 
-void
+static void
 config_monthly(pointers_t *pointers)
 {
   pointers->config->monthly_traffic = 1;
 } /* config_monthly */
 
 
-void
-config_html(pointers_t *pointers)
+static void
+config_size(pointers_t *pointers)
+{
+  pointers->config->size_traffic = 1;
+} /* config_size */
+
+
+static void
+config_html(pointers_t * pointers, char * html_location)
 {
   pointers->config->html_output = 1;
+
+  if (pointers->config->html_location != NULL)
+    g_free(pointers->config->html_location);
+
+  /* Add trailing '/' to html_location if it's not already there */
+  if (html_location[strlen(html_location) - 1] != '/')
+    {
+      pointers->config->html_location = g_malloc(strlen(html_location) + 2);
+      strcpy(pointers->config->html_location, html_location);
+      strcat(pointers->config->html_location, "/");
+    }
+  else
+    pointers->config->html_location = g_strdup(html_location);
 } /* config_html */
 
 
-void
+static void
 config_stdin(pointers_t *pointers)
 {
   pointers->config->use_stdin = 1;
 } /* config_stdin */
 
 
-void
+static void
 config_logtype(pointers_t *pointers, char *typestr)
 {
   int logtype;
 
   /* check text cases */
-  if (!strcasecmp(typestr, "wu-ftpd") || !strcasecmp(typestr, "wuftpd")
-      || !strcasecmp(typestr, "wu-ftp") || !strcasecmp(typestr, "wuftp"))
+  if (!g_strcasecmp(typestr, "wu-ftpd") || !g_strcasecmp(typestr, "wuftpd")
+      || !g_strcasecmp(typestr, "wu-ftp") || !g_strcasecmp(typestr, "wuftp"))
     {
       pointers->config->log_type = 0;
       return;
     }
-  else if (!strcasecmp(typestr, "ncftpd") || !strcasecmp(typestr, "ncftp"))
+  else if (!g_strcasecmp(typestr, "ncftpd") || !g_strcasecmp(typestr, "ncftp"))
     {
       pointers->config->log_type = 1;
       return;
     }
-  else if (!strcasecmp(typestr, "apache"))
+  else if (!g_strcasecmp(typestr, "apache"))
     {
       pointers->config->log_type = 2;
       return;
@@ -282,7 +333,7 @@ config_logtype(pointers_t *pointers, char *typestr)
 } /* config_logtype */
 
 
-void
+static void
 config_dir_depth(pointers_t *pointers, char *depthstr)
 {
   int depth = atoi(depthstr);
@@ -302,32 +353,36 @@ config_dir_depth(pointers_t *pointers, char *depthstr)
 } /* config_dir_depth */
 
 
-void
+static void
 config_dir_filter(pointers_t * pointers, char * path)
 {
   only_dir_t * item;
 
-  MY_MALLOC(item, sizeof(only_dir_t));
+  item = g_malloc(sizeof(only_dir_t));
 
-  MY_MALLOC(item->dir, (item->len = strlen(path)) + 1);
+  item->dir = g_malloc((item->len = strlen(path)) + 1);
   strcpy(item->dir, path);
 
-  list_add_item(&pointers->config->only_dir, item);
+  pointers->config->only_dir = g_slist_append(pointers->config->only_dir,
+					      item);
 } /* config_dir_filter */
 
 
+#if 0
 /* This will have to wait for another version */
-/*void
+static void
 config_tld_filter(pointers_t *pointers, char *tld)
 {
   if (pointers->config->only_tld != NULL)
-    free(pointers->config->only_tld);
-  pointers->config->only_tld = (char *)malloc((pointers->config->only_tld_length = strlen(tld)) + 1);
-  strcpy(pointers->config->only_tld, tld);
-} */ /* config_tld_filter */
+    g_free(pointers->config->only_tld);
+
+  pointers->config->only_tld = g_strdup(tld);
+  pointers->config->only_tld_length = strlen(tld);
+} /* config_tld_filter */
+#endif
 
 
-void
+static void
 config_number_file_stats(pointers_t *pointers, char *numstr)
 {
   long number = atoi(numstr);
@@ -347,7 +402,7 @@ config_number_file_stats(pointers_t *pointers, char *numstr)
 } /* config_number_file_stats */
 
 
-void
+static void
 config_number_dir_stats(pointers_t *pointers, char *numstr)
 {
   long number = atoi(numstr);
@@ -367,7 +422,7 @@ config_number_dir_stats(pointers_t *pointers, char *numstr)
 } /* config_number_dir_stats */
 
 
-void
+static void
 config_number_daily_stats(pointers_t *pointers, char *numstr)
 {
   long number = atoi(numstr);
@@ -387,7 +442,7 @@ config_number_daily_stats(pointers_t *pointers, char *numstr)
 } /* config_number_daily_stats */
 
 
-void
+static void
 config_number_tld_stats(pointers_t *pointers, char *numstr)
 {
   long number = atoi(numstr);
@@ -407,7 +462,7 @@ config_number_tld_stats(pointers_t *pointers, char *numstr)
 } /* config_number_tld_stats */
 
 
-void
+static void
 config_number_domain_stats(pointers_t *pointers, char *numstr)
 {
   long number = atoi(numstr);
@@ -428,7 +483,7 @@ config_number_domain_stats(pointers_t *pointers, char *numstr)
 } /* config_number_domain_stats */
 
 
-void
+static void
 config_number_host_stats(pointers_t *pointers, char *numstr)
 {
   long number = atoi(numstr);
@@ -448,7 +503,7 @@ config_number_host_stats(pointers_t *pointers, char *numstr)
 } /* config_number_host_stats */
 
 
-void
+static void
 config_max_report_size(pointers_t *pointers, char *numstr)
 {
   long number = atoi(numstr);
@@ -468,36 +523,159 @@ config_max_report_size(pointers_t *pointers, char *numstr)
 } /* config_max_report_size */
 
 
-void
-config_graph_path(pointers_t *pointers, char *path)
+static void
+config_graph_path(pointers_t * pointers, char * path)
 {
   if (pointers->config->graph_path)
-    free(pointers->config->graph_path);
+    g_free(pointers->config->graph_path);
 
-  pointers->config->graph_path = (char *)malloc(strlen(path) + 1);
-  strcpy(pointers->config->graph_path, path);
+  pointers->config->graph_path = g_strdup(path);
 } /* config_graph_path */
 
 
-void
+static void
 config_no_html_headers(pointers_t *pointers)
 {
   pointers->config->no_html_headers = 1;
 } /* config_no_html_headers */
 
 
-void
+static void
 config_single_page(pointers_t *pointers)
 {
   pointers->config->single_page = 1;
 } /* config_single_page */
 
 
+static void
+config_link_prefix(pointers_t * pointers, char * prefix)
+{
+  if (pointers->config->link_prefix != NULL)
+    g_free(pointers->config->link_prefix);
+
+  pointers->config->link_prefix = g_strdup(prefix);
+} /* config_link_prefix */
+
+
+static void
+config_additive_db(pointers_t * pointers, char * path)
+{
+  if (pointers->config->additive_db != NULL)
+    g_free(pointers->config->additive_db);
+
+  /* Add trailing '/' to the path if it's not already there */
+  if (path[strlen(path) - 1] != '/')
+    {
+      pointers->config->additive_db = g_malloc(strlen(path) + 2);
+      strcpy(pointers->config->additive_db, path);
+      strcat(pointers->config->additive_db, "/");
+    }
+  else
+    pointers->config->additive_db = g_strdup(path);
+} /* config_additive_db */
+
+
+static void
+config_strip_prefix(pointers_t * pointers, char * prefix)
+{
+  if (pointers->config->strip_prefix != NULL)
+    g_free(pointers->config->strip_prefix);
+
+  pointers->config->strip_prefix =
+    g_malloc((pointers->config->strip_prefix_len = strlen(prefix)) + 1);
+  strcpy(pointers->config->strip_prefix, prefix);
+} /* config_strip_prefix */
+
+
+static void
+config_file_sort_pref(pointers_t * pointers, char * pref)
+{
+  pointers->config->file_sort_pref = (char) atoi(pref);
+} /* config_file_sort_pref */
+
+
+static void
+config_dir_sort_pref(pointers_t * pointers, char * pref)
+{
+  pointers->config->dir_sort_pref = (char) atoi(pref);
+} /* config_dir_sort_pref */
+
+
+static void
+config_domain_sort_pref(pointers_t * pointers, char * pref)
+{
+  pointers->config->domain_sort_pref = (char) atoi(pref);
+} /* config_domain_sort_pref */
+
+
+static void
+config_tld_sort_pref(pointers_t * pointers, char * pref)
+{
+  pointers->config->tld_sort_pref = (char) atoi(pref);
+} /* config_tld_sort_pref */
+
+
+static void
+config_host_sort_pref(pointers_t * pointers, char * pref)
+{
+  pointers->config->host_sort_pref = (char) atoi(pref);
+} /* config_host_sort_pref */
+
+
+static void
+config_dom_sort_pref(pointers_t * pointers, char * pref)
+{
+  pointers->config->dom_sort_pref = (char) atoi(pref);
+} /* config_dom_sort_pref */
+
+
+static void
+config_dow_sort_pref(pointers_t * pointers, char * pref)
+{
+  pointers->config->dow_sort_pref = (char) atoi(pref);
+} /* config_dow_sort_pref */
+
+
+static void
+config_hourly_sort_pref(pointers_t * pointers, char * pref)
+{
+  pointers->config->hourly_sort_pref = (char) atoi(pref);
+} /* config_hourly_sort_pref */
+
+
+static void
+config_daily_sort_pref(pointers_t * pointers, char * pref)
+{
+  pointers->config->daily_sort_pref = (char) atoi(pref);
+} /* config_daily_sort_pref */
+
+
+static void
+config_size_sort_pref(pointers_t * pointers, char * pref)
+{
+  pointers->config->size_sort_pref = (char) atoi(pref);
+} /* config_size_sort_pref */
+
+
+static void
+config_monthly_sort_pref(pointers_t * pointers, char * pref)
+{
+  pointers->config->monthly_sort_pref = (char) atoi(pref);
+} /* config_monthly_sort_pref */
+
+
+static void
+config_strict_check(pointers_t *pointers)
+{
+  pointers->config->strict_check = 1;
+} /* config_strict_check */
+
+
 /* parse_config_line returns 0 if there is no valid data, 1 if there is only a
  * valid "command", 2 if there is a valid "command" and a valid "arg1", 3 if
  * there is also a valid "arg2", and 4 if the command and all 3 args are
  * valid */
-int
+static int
 parse_config_line(char *line, char **command, char *args[3])
 {
   int arg_no = 0;
